export const leaflet = (
  selection,
  { country, cities, toggleState, svg },
) => {
  svg
    .append('defs')
    .append('marker')
    .attr('id', 'pin')
    .attr('markerWidth', 10)
    .attr('markerHeight', 10)
    .attr('refX', 5)
    .attr('refY', 10)
    .attr('orient', 'auto')
    .append('path')
    .attr('d', 'M0,0 L5,10 L10,0 C5,2.5 5,2.5 0,0')
    .style('fill', '#213a1b');

  var mapB = L.map('app').setView([37.8, -96], 4);

  let osmLayer = L.tileLayer(
    'https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png',
    {
      attribution:
        '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors',
    },
  ).addTo(mapB);

  //initialize svg to add to map
  L.svg({ clickable: true }).addTo(mapB); // we have to make the svg layer clickable
  //Create selection using D3
  const overlay = d3.select(mapB.getPanes().overlayPane);
  const svgA = overlay
    .select('svg')
    .attr('pointer-events', 'auto');
  // create a group that is hidden during zooming
  const g = svgA
    .append('g')
    .attr('class', 'leaflet-zoom-hide');

  // Use Leaflets projection API for drawing svg path (creates a stream of projected points)
  const projectPoint = function (x, y) {
    const point = mapB.latLngToLayerPoint(
      new L.LatLng(y, x),
    );
    this.stream.point(point.x, point.y);
  };

  // Use d3's custom geo transform method to implement the above
  const projection = d3.geoTransform({
    point: projectPoint,
  });
  // creates geopath from projected points (SVG)
  const pathCreator = d3.geoPath().projection(projection);

  // selection
  // .selectAll('path.country')
  // .data(country.features)
  // .join('path')
  // .attr('d', path)
  // .attr('class', 'country');

  const areaPaths = g
    .selectAll('path.c')
    .data(country.features)
    .join('path')
    .attr('d', pathCreator)
    .attr('class', 'c')
    .attr('fill-opacity', 0.3)
    .attr('stroke', 'black')
    .attr('z-index', 3000)
    .attr('stroke-width', 2.5);

  // Function to place svg based on zoom
  const onZoom = () => areaPaths.attr('d', pathCreator);
  // initialize positioning
  onZoom();
  // reset whenever map is moved
  mapB.on('zoomend', onZoom);
};
