import {
  geoAlbersUsa,
  geoPath,
  geoGraticule,
  select,
} from 'd3';

const projection =
  geoAlbersUsaTerritories.geoAlbersUsaTerritories();

const path = geoPath(projection);
const graticule = geoGraticule();

export const map = (
  selection,
  { country, cities, toggleState },
) => {
  selection
    .selectAll('path.graticule')
    .data([null])
    .join('path')
    .attr('class', 'graticule')
    .attr('d', path(graticule()))
    .attr('fill', 'none')
    .attr('stroke', '#BBB')
    .attr('stroke-width', 0.2);

  const tooltip = select('body')
    .append('div')
    .attr('class', 'tooltip')
    .style('opacity', 0);

  selection
    .selectAll('path.country')
    .data(country.features)
    .join('path')
    .attr('d', path)
    .attr('class', 'country');

  for (const d of cities) {
    const [x, y] = projection([d.lng, d.lat]);
    d.x = x;
    d.y = y;
    d.visitorDifference = Math.round(
      ((d['2023'] - d['2022']) / d['2022']) * 100,
    );
  }

  const colorScale = d3
    .scaleSequential(
      d3.interpolateRgb('#ffba00', '#6f4930'),
    )
    .domain([0, d3.max(cities, (d) => d['2023'])]);

  selection
    .selectAll('circle.park')
    .data(cities)
    .join(
      (enter) =>
        enter
          .append('circle')
          .attr('cx', (d) => d.x)
          .attr('cy', (d) => d.y)
          .attr('fill', '#213a1b')
          .attr('r', 6)
          .attr('opacity', 1),

      (update) =>
        update.attr('fill', (d) =>
          toggleState ? colorScale(d['2023']) : '#213a1b',
        ),
      (exit) => exit.remove(),
    )
    .attr('class', 'park')
    .on('mouseover', function (event, d) {
      select(this).attr('opacity', '.4');
      tooltip
        .transition()
        .duration(1000)
        .style('opacity', 0.9);
      tooltip
        .html(function () {
          const visitorChange = d['visitorDifference'];
          const arrowIcon =
            visitorChange > 0
              ? '▲'
              : visitorChange < 0
                ? '▼'
                : '';
          const arrowColor =
            visitorChange > 0
              ? 'green'
              : visitorChange < 0
                ? 'red'
                : '';

          return `
      <div style="font-family: Arial, sans-serif; font-size: 14px; padding: 8px; background-color: rgba(255, 255, 255, 0.9); color: #333; border-radius: 5px; box-shadow: 0 2px 4px rgba(0, 0, 0, 0.1);">
        <strong>Park:</strong> ${d['Name']} <br>
        <strong>2023 Visitors:</strong> ${d['2023'].toLocaleString()} <br>
        <strong class="visitor-change">YoY Visitor Change: <span style="color: ${arrowColor};">${arrowIcon} ${visitorChange}%</span></strong> <br>
        <strong>Date established as National Park:</strong> ${d['date']}<br>
        <strong>Area:</strong> ${d['area']}
      </div>`;
        })
        .style('position', 'fixed')
        .style('pointer-events', 'none');

      // Adjust tooltip position to avoid it being cut off by screen edges
      const tooltipLeft =
        (event.pageY + 50 > window.innerHeight ||
          event.pageY < 100) &&
        event.pageX + 155 >= window.innerWidth - 100
          ? event.pageX - 350
          : Math.min(
              event.pageX + 10,
              window.innerWidth - 60,
            );

      const tooltipTop =
        event.pageY + 50 > window.innerHeight &&
        event.pageX + 100 > window.innerWidth - 200
          ? event.pageY - 100
          : Math.min(
              event.pageY + 10,
              window.innerHeight - 110,
            );

      tooltip
        .style('left', tooltipLeft + 'px')
        .style('top', tooltipTop + 'px');
    })
    // Hide tooltip on mouseout
    .on('mouseout', function () {
      select(this).attr('opacity', '1');
      tooltip.interrupt().style('opacity', 0);
    });

  selection
    .selectAll('path.outline')
    .data([null])
    .join('path')
    .attr('class', 'outline')
    .attr('d', path(graticule.outline()))
    .attr('fill', 'none')
    .attr('stroke', 'white')
    .attr('stroke-width', 0.5);